@@ -10,6 +10,7 @@ from sen.constants import ISO_DATETIME_PARSE_STRING
 from sen.exceptions import TerminateApplication, NotifyError, NotAvailableAnymore
 
 import docker
+import docker.errors
 
 from sen.net import NetData
 from sen.util import calculate_cpu_percent, calculate_blkio_bytes, calculate_network_bytes, repeater, \
@@ -18,6 +19,9 @@ from sen.util import calculate_cpu_percent, calculate_blkio_bytes, calculate_net
 logger = logging.getLogger(__name__)
 
 
+DINOSAUR_TIME = datetime.datetime.fromordinal(1)
+
+
 class ImageNameStruct(object):
     """
     stolen from atomic-reactor; thanks @mmilata!
@@ -174,6 +178,12 @@ class DockerObject:
             except KeyError:
                 raise RuntimeError("initial data not specified")
 
+    @property
+    def object_id(self):
+        if self._id is None:
+            self.set_id()
+        return self._id
+
     @property
     def short_id(self):
         if self._short_id is None:
@@ -224,6 +234,10 @@ class DockerObject:
             raise NotAvailableAnymore()
         return value
 
+    def refresh(self):
+        """ refresh metadata """
+        self.inspect(cached=False)
+
     def __eq__(self, other):
         return type(self) == type(other) and self._id == other._id
 
@@ -249,9 +263,7 @@ def graceful_chain_get(d, *args):
 class DockerImage(DockerObject):
     @property
     def image_id(self):
-        if self._id is None:
-            self.set_id()
-        return self._id
+        return self.object_id
 
     @property
     def parent_id(self):
@@ -294,7 +306,11 @@ class DockerImage(DockerObject):
         #     "Comment": "",
         #     "CreatedBy": "/bin/sh -c #(nop) ADD file:bcb5e5c... in /"
         # }
-        response = self.d.history(self.image_id)
+        try:
+            response = self.d.history(self.image_id)
+        except docker.errors.NotFound:
+            raise NotAvailableAnymore()
+
         layers = []
         for l in response:
             layer_id = l["Id"]
@@ -358,12 +374,21 @@ class DockerImage(DockerObject):
     @property
     def size(self):
         """
-        Size of all layers in bytes
+        Size of ALL layers in bytes
 
         :return: int
         """
         return self.data.get("VirtualSize", 0)
 
+    @property
+    def layer_size(self):
+        """
+        Size of ONLY this particular layer
+
+        :return: int
+        """
+        return self.data.get("Size", 0)
+
     @property
     def names(self):
         if self._names is None:
@@ -411,7 +436,10 @@ class DockerImage(DockerObject):
     @operation("Inspect image {object_short_name}.")
     def inspect(self, cached=False):
         if self._inspect is None or cached is False:
-            self._inspect = self.d.inspect_image(self.image_id)
+            try:
+                self._inspect = self.d.inspect_image(self.image_id)
+            except docker.errors.NotFound:
+                self._inspect = self._inspect or {}
         return self._inspect
 
     @operation("{object_type} {object_short_name} removed!")
@@ -480,9 +508,7 @@ class DockerContainer(DockerObject):
 
     @property
     def container_id(self):
-        if self._id is None:
-            self.set_id()
-        return self._id
+        return self.object_id
 
     @property
     def names(self):
@@ -499,20 +525,32 @@ class DockerContainer(DockerObject):
         return self.data["Command"]
 
     @property
-    def status(self):
+    def nice_status(self):
         return self.data["Status"]
 
+    @property
+    def simple_status(self):
+        return self.metadata_get(["State", "Status"])
+
+    @property
+    def simple_status_cap(self):
+        return self.simple_status.capitalize()
+
     @property
     def running(self):
-        return self.status.startswith("Up")
+        return self.metadata_get(["State", "Running"])
 
     @property
     def status_created(self):
-        return self.status.startswith("Created")
+        return self.simple_status == "created"
+
+    @property
+    def exit_code(self):
+        return self.metadata_get(["State", "ExitCode"])
 
     @property
     def exited_well(self):
-        return self.status.startswith("Exited (0)")
+        return self.exit_code == 0
 
     @property
     def short_name(self):
@@ -568,9 +606,8 @@ class DockerContainer(DockerObject):
             # python expects 6 digits in milliseconds, docker returns 9
             s = s[:26]
             if s == "0001-01-01T00:00:00Z":
-                return datetime.datetime.fromordinal(1)
+                return DINOSAUR_TIME
             started_at = datetime.datetime.strptime(s, ISO_DATETIME_PARSE_STRING)
-            logger.debug("started at %s", started_at)
             return started_at
 
     @property
@@ -579,9 +616,8 @@ class DockerContainer(DockerObject):
         if f:
             f = f[:26]
             if f == "0001-01-01T00:00:00Z":
-                return datetime.datetime.fromordinal(1)
+                return DINOSAUR_TIME
             finished_at = datetime.datetime.strptime(f, ISO_DATETIME_PARSE_STRING)
-            logger.debug("finished at %s", finished_at)
             return finished_at
 
     @property
@@ -589,7 +625,13 @@ class DockerContainer(DockerObject):
         # docker acts weird: 'created' is provided as ordinal and is local time
         # 'started' and 'finished' are UTC as timestamp; it would be awesome to unite those b/c
         # atm these inconsistencies mess ordering
-        return max(self.started_at, self.finished_at, super().natural_sort_value)
+        try:
+            # Nones are unsortable
+            return max([x for x in
+                        [self.started_at, self.finished_at, super().natural_sort_value]
+                        if x])
+        except NotAvailableAnymore:
+            return super().natural_sort_value
 
     # methods
 
@@ -630,13 +672,15 @@ class DockerContainer(DockerObject):
 
         :return: None or list of dicts
         """
-        if not self.running:
-            return []
         # let's get resources from .stats()
         ps_args = "-eo pid,ppid,wchan,args"
         # returns {"Processes": [values], "Titles": [values]}
         # it's easier to play with list of dicts: [{"pid": 1, "ppid": 0}]
-        response = self.d.top(self.container_id, ps_args=ps_args)
+        try:
+            response = self.d.top(self.container_id, ps_args=ps_args)
+        except docker.errors.APIError as ex:
+            logger.warning("error getting processes: %r", ex)
+            return []
         # TODO: sort?
         logger.debug(json.dumps(response, indent=2))
         return [dict(zip(response["Titles"], process))
@@ -645,14 +689,22 @@ class DockerContainer(DockerObject):
     @operation("Inspect container {object_short_name}.")
     def inspect(self, cached=False):
         if cached is False or self._inspect is None:
-            self._inspect = self.d.inspect_container(self.container_id)
+            try:
+                self._inspect = self.d.inspect_container(self.container_id)
+            except docker.errors.NotFound:
+                self._inspect = self._inspect or {}
         return self._inspect
 
     @operation("Logs of container {object_short_name} received.")
     def logs(self, follow=False, lines="all"):
         # when tail is set to all, it takes ages to populate widget
-        logs_data = self.d.logs(self.container_id, stream=follow, tail=lines)
-        return logs_data
+        # docker-py does `inspect` in the background
+        try:
+            logs_data = self.d.logs(self.container_id, stream=follow, tail=lines)
+        except docker.errors.NotFound:
+            return None
+        else:
+            return logs_data
 
     @operation("{object_type} {object_short_name} removed!")
     def remove(self):
@@ -732,38 +784,40 @@ class DockerBackend:
         return list(self._containers.values())
 
     def realtime_updates(self):
-        it = repeater(self.client.events, kwargs={"decode": True}, retries=5)
+        event = it = None
         while True:
-            event = repeater(next, args=(it, ), retries=2)  # likely an engine restart
-            if not event:
+            if not it or not event:
                 it = repeater(self.client.events, kwargs={"decode": True}, retries=5)
                 if not it:
                     raise NotifyError("Unable to fetch realtime updates from docker engine.")
-                continue
-            logger.debug("RT event: %s", event)
-            yield event
-            continue
-            # FIXME: this needs rewrite
 
-            try:
-                # 1.10+
-                is_container = event["Type"] == "container"
-            except KeyError:
-                # event["from'] means it's a container
-                is_container = "from" in event
-            if is_container:
-                # inspect doesn't contain info about status and you can't query just one
-                # container with containers()
-                # let's do full-blown containers() query; it's not that expensive
-                self.get_containers(cached=False)
-            else:
-                # similar as ^
-                # images() doesn't support query by ID
-                # inspect doesn't contain info about repositories
-                self.get_images(cached=False)
-            content, _, _ = self.filter(containers=True, images=True, stopped=True,
-                                        cached=True, sort_by_created=True)
-            yield content
+            event = repeater(next, args=(it, ), retries=2)  # likely an engine restart
+            if not event:
+                continue
+
+            logger.debug("RT event: %s", event)
+
+            yield event
+
+            # try:
+            #     # 1.10+
+            #     is_container = event["Type"] == "container"
+            # except KeyError:
+            #     # event["from'] means it's a container
+            #     is_container = "from" in event
+            # if is_container:
+            #     # inspect doesn't contain info about status and you can't query just one
+            #     # container with containers()
+            #     # let's do full-blown containers() query; it's not that expensive
+            #     self.get_containers(cached=False)
+            # else:
+            #     # similar as ^
+            #     # images() doesn't support query by ID
+            #     # inspect doesn't contain info about repositories
+            #     self.get_images(cached=False)
+            # content, _, _ = self.filter(containers=True, images=True, stopped=True,
+            #                             cached=True, sort_by_created=True)
+            # yield content
 
     # service methods
 
