@@ -1,10 +1,7 @@
 import os
-import sys
 import logging
 import functools
-import threading
 import traceback
-
 import time
 from datetime import datetime
 
@@ -12,6 +9,7 @@ from docker.errors import APIError
 
 from sen.constants import PROJECT_NAME, LOG_FILE_NAME
 
+
 logger = logging.getLogger(__name__)
 
 
